@@ -10,6 +10,13 @@ import re
 import urllib
 from routes import request_config
 
+class literal(str):
+    """Literal class for use in systems to indicate the url is already
+    escaped"""
+    def __html__(self):
+        return self
+
+
 def _screenargs(kargs):
     """
     Private function that takes a dict, and screens it against the current 
@@ -17,6 +24,13 @@ def _screenargs(kargs):
     This is responsible for the requests "memory" of the current.
     """
     config = request_config()
+    
+    # Coerce any unicode args with the encoding
+    encoding = config.mapper.encoding
+    for key, val in kargs.iteritems():
+        if isinstance(val, unicode):
+            kargs[key] = val.encode(encoding)
+    
     if config.mapper.explicit and config.mapper.sub_domains:
         return _subdomain_check(config, kargs)
     elif config.mapper.explicit:
@@ -56,8 +70,17 @@ def _subdomain_check(config, kargs):
         subdomain = kargs.pop('sub_domain', None)
         if isinstance(subdomain, unicode):
             subdomain = str(subdomain)
-        fullhost = config.environ.get('HTTP_HOST') or \
-            config.environ.get('SERVER_NAME')
+        
+        # We use a try/except here, cause the only time there should be no
+        # environ is when we're unit testing, in which case we shouldn't be
+        # changing kargs and such. The exception catching also won't hurt as
+        # badly here vs doing a hasattr on every url check
+        try:
+            fullhost = config.environ.get('HTTP_HOST') or \
+                config.environ.get('SERVER_NAME')
+        except AttributeError:
+            return kargs
+        
         hostmatch = fullhost.split(':')
         host = hostmatch[0]
         port = ''
@@ -77,11 +100,18 @@ def _subdomain_check(config, kargs):
     
 
 def _url_quote(string, encoding):
-    """A Unicode handling version of urllib.quote_plus."""
+    """A Unicode handling version of urllib.quote."""
     if encoding:
-        return urllib.quote_plus(unicode(string).encode(encoding), '/')
+        if isinstance(string, unicode):
+            s = string.encode(encoding)
+        elif isinstance(string, str):
+            # assume the encoding is already correct
+            s = string
+        else:
+            s = unicode(string).encode(encoding)
     else:
-        return urllib.quote_plus(str(string), '/')
+        s = str(string)
+    return urllib.quote(s, '/')
 
 def url_for(*args, **kargs):
     """Generates a URL 
@@ -163,9 +193,15 @@ def url_for(*args, **kargs):
                 url += '?'
                 query_args = []
                 for key, val in kargs.iteritems():
-                    query_args.append("%s=%s" % (
-                        urllib.quote_plus(unicode(key).encode(encoding)),
-                        urllib.quote_plus(unicode(val).encode(encoding))))
+                    if isinstance(val, (list, tuple)):
+                        for value in val:
+                            query_args.append("%s=%s" % (
+                                urllib.quote(unicode(key).encode(encoding)),
+                                urllib.quote(unicode(value).encode(encoding))))
+                    else:
+                        query_args.append("%s=%s" % (
+                            urllib.quote(unicode(key).encode(encoding)),
+                            urllib.quote(unicode(val).encode(encoding))))
                 url += '&'.join(query_args)
     if not static:
         route_args = []
@@ -196,16 +232,19 @@ def url_for(*args, **kargs):
             host = config.host.split(':')[0]
         elif not host:
             host = config.host
-        if not protocol: 
+        if not protocol:
             protocol = config.protocol
         if url is not None:
             url = protocol + '://' + host + url
     
     if not isinstance(url, str) and url is not None:
-        raise Exception("url_for can only return a string or None, got "
+        raise RouteException("url_for can only return a string, got "
                         "unicode instead: %s" % url)
-    
-    return url
+    if url is None:
+        raise RouteException(
+            "url_for could not generate URL. Called with args: %s %s" % \
+            (args, kargs))
+    return literal(url)
 
 def redirect_to(*args, **kargs):
     """Issues a redirect based on the arguments. 
